unit mainform;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, XPMan, ComCtrls, ActnMan, ActnCtrls, ActnMenus,
  ActnList, XPStyleActnCtrls, ToolWin, ExtCtrls, ImgList, StdStyleActnCtrls,
	NoteFile, Menus, CustomizeDlg, ActnPopupCtrl, brwsfldr, AboutBox;

type
  TfrmMain = class(TForm)
    stbMain: TStatusBar;
    cbrMain: TCoolBar;
    acmMain: TActionManager;
    actFileOpen: TAction;
    ActionMainMenuBar1: TActionMainMenuBar;
    pnlToolbar: TPanel;
		splMain: TSplitter;
    imlTreeView: TImageList;
    dlgOpen: TOpenDialog;
		actFileExit: TAction;
		actViewFontMemo: TAction;
    dlgFont: TFontDialog;
    actHelpAbout: TAction;
    actEditFind: TAction;
    dlgFind: TFindDialog;
    actEditSelectAllTreeView: TAction;
    imlToolbutton: TImageList;
    actEditCopyTreeView: TAction;
    pumTreeView: TPopupActionBarEx;
    idpEditCopyTreeView: TMenuItem;
    pumMemo: TPopupActionBarEx;
    idpEditCopy: TMenuItem;
    idpEditSelectAll: TMenuItem;
    actFileExportTreeView: TAction;
		tvwMain: TTreeView;
		pnlRightPane: TPanel;
		memMain: TMemo;
		lvwMain: TListView;
		actFileClose: TAction;
		actFileExportListView: TAction;
		pumListView: TPopupActionBarEx;
		idpFileExportMulti: TMenuItem;
		idpFileExportTreeView: TMenuItem;
		dlgSave: TSaveDialog;
		actViewFontTreeView: TAction;
		dlgBrowseForFolder: TBrowseForFolderDialog;
		dlgAbout: TAboutBox;
		procedure FormCreate(Sender: TObject);
		procedure FormDestroy(Sender: TObject);
		procedure actFileOpenExecute(Sender: TObject);
		procedure lvwMainDataFind(Sender: TObject; Find: TItemFind; const FindString: String; const FindPosition: TPoint;FindData: Pointer; StartIndex: Integer; Direction: TSearchDirection; Wrap: Boolean; var Index: Integer);
		procedure actFileExitExecute(Sender: TObject);
		procedure lvwMainSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
		procedure actViewFontMemoExecute(Sender: TObject);
		procedure actHelpAboutExecute(Sender: TObject);
		procedure actEditFindUpdate(Sender: TObject);
		procedure actEditFindExecute(Sender: TObject);
		procedure dlgFindFind(Sender: TObject);
		procedure actEditSelectAllTreeViewUpdate(Sender: TObject);
		procedure actEditSelectAllTreeViewExecute(Sender: TObject);
		procedure actEditCopyTreeViewUpdate(Sender: TObject);
		procedure actEditCopyTreeViewExecute(Sender: TObject);
		procedure actFileExportTreeViewUpdate(Sender: TObject);
		procedure tvwMainChange(Sender: TObject; Node: TTreeNode);
		procedure tvwMainMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
		procedure actFileCloseExecute(Sender: TObject);
		procedure actFileCloseUpdate(Sender: TObject);
		procedure lvwMainData(Sender: TObject; Item: TListItem);
		procedure actFileExportListViewUpdate(Sender: TObject);
		procedure actFileExportTreeViewExecute(Sender: TObject);
		procedure actFileExportListViewExecute(Sender: TObject);
		procedure actViewFontTreeViewExecute(Sender: TObject);
		procedure actViewFontTreeViewUpdate(Sender: TObject);
		procedure actViewFontMemoUpdate(Sender: TObject);
		procedure lvwMainDblClick(Sender: TObject);
	private
		FObjectTextFile: TObjectTextFile;
		procedure FormUpdateCaption(Sender: TObject);
		procedure FormUpdateStatus(Sender: TObject);
		procedure FormUpdateTree(Sender: TObject);
		procedure FormInitialize(Sender: TObject);
		procedure FormFinalize(Sender: TObject);
		procedure FormOpenFile(Sender: TObject; FileName: string);
		procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
	public
    procedure ReportProgress(Sender: TObject; Stage: TProgressStage; PercentDone: byte);
    property ObjectTextFile: TObjectTextFile read FObjectTextFile;
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

uses
  Clipbrd,
  INIFiles,
	ShellAPI;

type
  TIconIndex = (iiFile, iiTable, iiForm, iiReport, iiDataport, iiCodeunit);

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FormInitialize(Self);

  FObjectTextFile := TObjectTextFile.Create;
  ObjectTextFile.OnProgress := ReportProgress;

  if ParamCount > 0 then
  begin
    Show;
    FormOpenFile(Self, ParamStr(1));
  end
  else
  begin
    FormUpdateCaption(Self);
    FormUpdateStatus(Self);
  end;

  DragAcceptFiles(Handle, true);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  FormFinalize(Self);
  FObjectTextFile.Free;
end;

procedure TfrmMain.FormUpdateCaption(Sender: TObject);
begin
  Application.Title := sAppTitle;

  if ObjectTextFile.FileName <> '' then
    Application.Title := Format('%s - %s', [ExtractFileName(ObjectTextFile.FileName), Application.Title]);

  Caption := Application.Title;
end;

procedure TfrmMain.FormUpdateStatus(Sender: TObject);
resourcestring
  sObjectCount = '%d object(s)';
begin
  stbMain.SimpleText := Format(sObjectCount, [ObjectTextFile.Count]);
end;

procedure TfrmMain.ReportProgress(Sender: TObject; Stage: TProgressStage;
  PercentDone: byte);
resourcestring
  sPercentDone = 'Processing File... %d%% done';
begin
  case Stage of
    psRunning: stbMain.SimpleText := Format(sPercentDone, [PercentDone]);
    psEnding: FormUpdateStatus(Self);
  end;
end;

procedure TfrmMain.actFileOpenExecute(Sender: TObject);
begin
  if dlgOpen.Execute then
    FormOpenFile(Self, dlgOpen.FileName);
end;

procedure TfrmMain.lvwMainDataFind(Sender: TObject; Find: TItemFind;
  const FindString: String; const FindPosition: TPoint; FindData: Pointer;
  StartIndex: Integer; Direction: TSearchDirection; Wrap: Boolean;
  var Index: Integer);
var
  i: integer;
begin
  for i := StartIndex to ObjectTextFile.Count - 1 do
    if LowerCase(Copy(ObjectTextFile.Objects[i].Signature, 1, Length(FindString))) = LowerCase(FindString) then
    begin
      Index := i;
      exit;
    end;

  if Wrap and (StartIndex > 0) then
    for i := 0 to StartIndex - 1 do
      if LowerCase(Copy(ObjectTextFile.Objects[i].Signature, 1, Length(FindString))) = LowerCase(FindString) then
      begin
        Index := i;
        exit;
      end;

  Index := -1;
end;

procedure TfrmMain.actFileExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.lvwMainSelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  if Assigned(Item) then
    with ObjectTextFile.Objects[Item.Index] do
      memMain.Text := Text;
end;

procedure TfrmMain.actViewFontMemoExecute(Sender: TObject);
begin
  dlgFont.Font := memMain.Font;

  if dlgFont.Execute then
    memMain.Font := dlgFont.Font;
end;

resourcestring
  sSectionWindow = 'Window';
  sSectionView = 'View';
  sSectionFind = 'Find';

  sKeyWidth = 'Width';
  sKeyHeight = 'Height';
  sKeyLeft = 'Left';
  sKeyTop = 'Top';
  
  sKeyListFontName = 'ListFontName';
  sKeyListFontSize = 'ListFontSize';
  sKeyListFontBold = 'ListFontBold';
  sKeyListFontItalic = 'ListFontItalic';

  sKeyCodeFontName = 'CodeFontName';
  sKeyCodeFontSize = 'CodeFontSize';
  sKeyCodeFontBold = 'CodeFontBold';
  sKeyCodeFontItalic = 'CodeFontItalic';

  sKeyObjListWidth = 'ObjectListWidth';
  sKeyFindText = 'FindText';
  sKeyMatchCase = 'MatchCase';

procedure TfrmMain.FormFinalize(Sender: TObject);
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(paramStr(0)) + 'note.zup');
  try
    if WindowState = wsNormal then
    begin
      IniFile.WriteInteger(sSectionWindow, sKeyWidth, Width);
      IniFile.WriteInteger(sSectionWindow, sKeyHeight, Height);
      IniFile.WriteInteger(sSectionWindow, sKeyLeft, Left);
      IniFile.WriteInteger(sSectionWindow, sKeyTop, Top);
    end;

    IniFile.WriteString(sSectionView, sKeyCodeFontName, memMain.Font.Name);
    IniFile.WriteInteger(sSectionView, sKeyCodeFontSize, memMain.Font.Size);
    IniFile.WriteInteger(sSectionView, sKeyCodeFontBold, integer(fsBold in memMain.Font.Style));
    IniFile.WriteInteger(sSectionView, sKeyCodeFontItalic, integer(fsItalic in memMain.Font.Style));

    IniFile.WriteString(sSectionView, sKeyListFontName, lvwMain.Font.Name);
    IniFile.WriteInteger(sSectionView, sKeyListFontSize, lvwMain.Font.Size);
    IniFile.WriteInteger(sSectionView, sKeyListFontBold, integer(fsBold in lvwMain.Font.Style));
    IniFile.WriteInteger(sSectionView, sKeyListFontItalic, integer(fsItalic in lvwMain.Font.Style));

    IniFile.WriteInteger(sSectionView, sKeyObjListWidth, tvwMain.Width);
    IniFile.WriteString(sSectionFind, sKeyFindText, dlgFind.FindText);
    IniFile.WriteInteger(sSectionFind, sKeyMatchCase, integer(frMatchCase in dlgFind.Options));
  finally
    IniFile.Free;
  end;
end;

procedure TfrmMain.FormInitialize(Sender: TObject);
var
  IniFile: TIniFile;
begin
  IniFile := TIniFile.Create(ExtractFilePath(paramStr(0)) + 'note.zup');
  try
    Width := IniFile.ReadInteger(sSectionWindow, sKeyWidth, Screen.Width div 2);
    Height := IniFile.ReadInteger(sSectionWindow, sKeyHeight, Screen.Height div 2);
    Left := IniFile.ReadInteger(sSectionWindow, sKeyLeft, (Screen.Width - Width) div 2);
    Top := IniFile.ReadInteger(sSectionWindow, sKeyTop, (Screen.Height - Height) div 2);
    tvwMain.Width := IniFile.Readinteger(sSectionView, sKeyObjListWidth, Width div 4);

    memMain.Font.Name := IniFile.ReadString(sSectionView, sKeyCodeFontName, 'Courier New');
    memMain.Font.Size := IniFile.ReadInteger(sSectionView, sKeyCodeFontSize, 9);
    if boolean(IniFile.ReadInteger(sSectionView, sKeyCodeFontBold, 0)) then
      memMain.Font.Style := memMain.Font.Style + [fsBold];
    if boolean(IniFile.ReadInteger(sSectionView, sKeyCodeFontItalic, 0)) then
      memMain.Font.Style := memMain.Font.Style + [fsItalic];

    lvwMain.Font.Name := IniFile.ReadString(sSectionView, sKeyListFontName, 'Tahoma');
    lvwMain.Font.Size := IniFile.ReadInteger(sSectionView, sKeyListFontSize, 8);
    if boolean(IniFile.ReadInteger(sSectionView, sKeyListFontBold, 0)) then
      lvwMain.Font.Style := lvwMain.Font.Style + [fsBold];
    if boolean(IniFile.ReadInteger(sSectionView, sKeyListFontItalic, 0)) then
      lvwMain.Font.Style := lvwMain.Font.Style + [fsItalic];

    tvwMain.Font := lvwMain.Font;

    dlgFind.FindText := IniFile.ReadString(sSectionFind, sKeyFindText, '');
    if boolean(IniFile.ReadInteger(sSectionFind, sKeyMatchCase, 0)) then
      dlgFind.Options := dlgFind.Options + [frMatchCase];
  finally
    IniFile.Free;
  end;
end;

procedure TfrmMain.WMDropFiles(var Msg: TWMDropFiles);
var
  Count: integer;
  FileName: array[0..Max_Path] of char;
begin
  Count := DragQueryFile(Msg.Drop, $FFFFFFFF, FileName, Max_Path);

  if Count > 0 then
  begin
    DragQueryFile(Msg.Drop, 0, FileName, Max_Path);
    FormOpenFile(Self, FileName);
  end;

  DragFinish(Msg.Drop);
end;

procedure TfrmMain.actHelpAboutExecute(Sender: TObject);
begin
	dlgAbout.Execute;
end;

procedure TfrmMain.FormOpenFile(Sender: TObject; FileName: string);
resourcestring
  sReadingFile = 'Reading File...';
begin
  stbMain.SimpleText := sReadingFile;
  Screen.Cursor := crHourglass;
  try
    Application.ProcessMessages;
    ObjectTextFile.LoadFromFile(FileName);
    FormUpdateTree(Self);
    FormUpdateCaption(Self);
    tvwMain.Selected := tvwMain.Items[0];
    lvwMain.Items.Count := ObjectTextFile.Count;
    lvwMain.Repaint;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.actEditFindUpdate(Sender: TObject);
begin
  actEditFind.Enabled := (ObjectTextFile.FileName <> '');
end;

procedure TfrmMain.actEditFindExecute(Sender: TObject);
begin
  dlgFind.Execute;
end;

procedure TfrmMain.dlgFindFind(Sender: TObject);
var
  ObjectIndex: integer;
  Offset: integer;
begin
  Screen.Cursor := crHourglass;
  try
    case tvwMain.Selected.AbsoluteIndex = 0 of
      true: ObjectIndex := 0;
      false: ObjectIndex := tvwMain.Selected.Index;
    end;

    Offset := memMain.SelStart + memMain.SelLength + 1;

    if ObjectTextFile.FindText(ObjectIndex, Offset, dlgFind.FindText, frMatchCase in dlgFind.Options) then
    begin
      tvwMain.Items[ObjectIndex + 1].Selected := true;
      tvwMain.Items[ObjectIndex + 1].MakeVisible;
      memMain.SelStart := Offset - 1;
      memMain.SelLength := Length(dlgFind.FindText);
    end
    else Beep;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.actEditSelectAllTreeViewUpdate(Sender: TObject);
begin
  actEditSelectAllTreeView.Enabled := (ObjectTextFile.FileName <> '');
end;

procedure TfrmMain.actEditSelectAllTreeViewExecute(Sender: TObject);
begin
  if ActiveControl is TListView then
  begin
    lvwMain.SelectAll;
    exit;
  end;

  if ActiveControl is TMemo then
  begin
    memMain.SelectAll;
    exit;
  end;

  if ActiveControl is TTreeView then
  begin
    if tvwMain.Selected.AbsoluteIndex = 0 then
      lvwMain.SelectAll
    else
      memMain.SelectAll;
  end;
end;

procedure TfrmMain.actEditCopyTreeViewUpdate(Sender: TObject);
begin 
  actEditCopyTreeView.Enabled := false;

	if tvwMain.Selected <> nil then
    if tvwMain.Selected.AbsoluteIndex <> 0 then
      actEditCopyTreeView.Enabled := true;
end;

procedure TfrmMain.actEditCopyTreeViewExecute(Sender: TObject);
begin
  if memMain.SelLength = 0 then
    Clipboard.AsText := memMain.Text
  else
    Clipboard.AsText := memMain.SelText;
end;

procedure TfrmMain.actFileExportTreeViewUpdate(Sender: TObject);
begin
  actFileExportTreeView.Enabled :=
    (ActiveControl is TTreeView) and
    (tvwMain.Items.Count > 0) and
    (tvwMain.Selected.AbsoluteIndex <> 0);
  actFileExportTreeView.Visible :=
    ActiveControl is TTreeView;
end;

procedure TfrmMain.tvwMainChange(Sender: TObject; Node: TTreeNode);
begin
  if Node.AbsoluteIndex > 0 then
  begin
    memMain.Visible := true;
    lvwMain.Visible := false;
    with ObjectTextFile.Objects[Node.Index] do
      memMain.Text := Text;
  end
  else
  begin
    memMain.Visible := false;
    lvwMain.Visible := true;
  end;
end;

procedure TfrmMain.FormUpdateTree(Sender: TObject);
var
  i: integer;
  Root, Child: TTreeNode;
begin
  tvwMain.Items.BeginUpdate;
  try
    Screen.Cursor := crHourglass;
    tvwMain.Items.Clear;

		if ObjectTextFile.FileName <> '' then
    begin
      Root := tvwMain.Items.Add(nil, ExtractFileName(ObjectTextFile.FileName));
      Root.ImageIndex := integer(iiFile);

      for i := 0 to ObjectTextFile.Count - 1 do
        with ObjectTextFile.Objects[i] do
          begin
            Child := tvwMain.Items.AddChild(Root, Signature);
            Child.ImageIndex := integer(ObjectType);
            Child.SelectedIndex := integer(ObjectType);
          end;

      tvwMain.Selected := Root;
      if ObjectTextFile.Count > 0 then
        Root.Expand(false);
    end;
  finally
    Screen.Cursor := crDefault;
    tvwMain.Items.EndUpdate;
  end;
end;

procedure TfrmMain.tvwMainMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if tvwMain.GetNodeAt(X, Y) <> nil then
    tvwMain.Selected := tvwMain.GetNodeAt(X, Y);
end;

procedure TfrmMain.actFileCloseExecute(Sender: TObject);
begin
  ObjectTextFile.Clear;
  memMain.Clear;
  lvwMain.Items.Count := 0;
  lvwMain.Clear;

  FormUpdateTree(Self);
  FormUpdateCaption(Self);
  FormUpdateStatus(Self);
end;

procedure TfrmMain.actFileCloseUpdate(Sender: TObject);
begin
  actFileClose.Enabled := (ObjectTextFile.FileName <> '');
end;

procedure TfrmMain.lvwMainData(Sender: TObject; Item: TListItem);
begin
	with ObjectTextFile.Objects[Item.Index] do
	begin
		Item.Caption := sObjectType[ObjectType];
		Item.SubItems.Add(IntToStr(ObjectID));
		Item.SubItems.Add(ObjectName);
		Item.ImageIndex := integer(ObjectType);
  end;
end;

procedure TfrmMain.actFileExportListViewUpdate(Sender: TObject);
begin
  actFileExportListView.Visible := (ActiveControl is TListView);
  actFileExportListView.Enabled := (lvwMain.SelCount > 0);
end;

procedure TfrmMain.actFileExportTreeViewExecute(Sender: TObject);
begin
  with ObjectTextFile.Objects[tvwMain.Selected.Index] do
  begin
    dlgSave.FileName := DefaultFileName;
    if dlgSave.Execute then
      SaveToFile(dlgSave.FileName);
  end;
end;

procedure TfrmMain.actFileExportListViewExecute(Sender: TObject);
resourcestring
  sFileExists = '%s already exists.'#13#10'Do you want to replace it?';
var
  ReplaceAll: boolean;
  ListItem: TListItem;
  FileName: string;
begin
  case lvwMain.SelCount of
    0: exit;
    1:
      begin
        with ObjectTextFile.Objects[lvwMain.Selected.Index] do
        begin
          dlgSave.FileName := DefaultFileName;
          if dlgSave.Execute then
            SaveToFile(dlgSave.FileName);
        end;
      end;
  else
		if dlgBrowseForFolder.Execute then
    begin
      ReplaceAll := false;
      ListItem := lvwMain.Selected;
			while ListItem <> nil do
			begin
				with ObjectTextFile.Objects[ListItem.Index] do
				begin
					FileName :=  IncludeTrailingPathDelimiter(dlgBrowseForFolder.FolderName) + DefaultFileName;

          case FileExists(FileName) of
            false: SaveToFile(FileName);
            true:
              case ReplaceAll of
                true: SaveToFile(FileName);
                false:
                  case MessageDlg(Format(sFileExists, [FileName]), mtWarning, [mbYes, mbNo, mbCancel, mbYesToAll], 0) of
                    mrCancel: exit;
                    mrYes: SaveToFile(FileName);
                    mrYesToAll:
                      begin
                        SaveToFile(FileName);
                        ReplaceAll := true;
                      end;
                  end;
              end;
          end;
        end;

       ListItem := lvwMain.GetNextItem(ListItem, sdBelow, [isSelected]);
      end;
    end;
  end;
end;

procedure TfrmMain.actViewFontTreeViewExecute(Sender: TObject);
begin
  dlgFont.Font := tvwMain.Font;

  if dlgFont.Execute then
  begin
    tvwMain.Font := dlgFont.Font;
    lvwMain.Font := dlgFont.Font;
  end;
end;

procedure TfrmMain.actViewFontTreeViewUpdate(Sender: TObject);
begin
  actViewFontTreeView.Visible := (ActiveControl is TListView) or (ActiveControl is TTreeView);
end;

procedure TfrmMain.actViewFontMemoUpdate(Sender: TObject);
begin
  actViewFontMemo.Visible := (ActiveControl is TMemo);
end;

procedure TfrmMain.lvwMainDblClick(Sender: TObject);
begin
  tvwMain.Select(tvwMain.Items[lvwMain.Selected.Index + 1]);
end;

end.
